#include "app.hpp"
#include <iostream>
#include <giomm/menu.h>
#include <gtkmm/builder.h>
#include <assert.h>

namespace components {
    App::App()
        : Gtk::Application("com.github.aquadim.YTMPV")
    {
    }

    Glib::RefPtr<App> App::create()
    {
        auto app = Glib::make_refptr_for_instance<App>(new App());
        return app;
    }

    void App::on_startup()
    {
        Gtk::Application::on_startup();

        add_action("preferences", sigc::mem_fun(*this, &App::onActionPrefs));
        set_accel_for_action("app.preferences", "<Ctrl>P");

        add_action("quit", sigc::mem_fun(*this, &App::onActionQuit));
        set_accel_for_action("app.quit", "<Ctrl>Q");

        // Установка менюбара
        auto menu_builder = Gtk::Builder::create_from_resource("/src/YTMPV/res/MainMenu.ui");
        auto menu = menu_builder->get_object<Gio::Menu>("menu");
        assert(menu != nullptr);
        
        set_menubar(menu);
    }

    void App::on_activate()
    {
        auto appwindow = createMainWindow();
        appwindow->present();
    }

    MainWindow* App::createMainWindow()
    {
        // Создание окна и привязка обработчика к его сигналу скрытия
        auto appwindow = MainWindow::create();
        add_window(*appwindow);
        appwindow->signal_hide().connect(
            [appwindow](){ delete appwindow; });
        return appwindow;
    }

    PrefWindow* App::createPrefWindow()
    {
        auto prefwindow = PrefWindow::create(
            *get_active_window()
        );
        add_window(*prefwindow);
        prefwindow->signal_hide().connect(
            [prefwindow](){ delete prefwindow; });
        return prefwindow;
    }

    void App::onActionQuit()
    {
        auto windows = get_windows();
        for (auto window : windows)
        {
            window->set_visible(false);
        }
        quit();
    }
    
    void App::onActionPrefs()
    {
        PrefWindow* win = createPrefWindow();
        win->present();
    }
}